==> truffle-nfi-1.0.0-rc14-sources.jar: com/oracle/truffle/nfi/impl/ClosureArgumentNode.java <==
/*
 * Copyright (c) 2017, 2018, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * The Universal Permissive License (UPL), Version 1.0
 *
 * Subject to the condition set forth below, permission is hereby granted to any
 * person obtaining a copy of this software, associated documentation and/or
 * data (collectively the "Software"), free of charge and under any and all
 * copyright rights in the Software, and any and all patent rights owned or
 * freely licensable by each licensor hereunder covering either (i) the
 * unmodified Software as contributed to or provided by such licensor, or (ii)
 * the Larger Works (as defined below), to deal in both
 *
 * (a) the Software, and
 *
 * (b) any piece of software and/or hardware listed in the lrgrwrks.txt file if
 * one is included with the Software each a "Larger Work" to which the Software
 * is contributed by such licensors),
 *
 * without restriction, including without limitation the rights to copy, create
 * derivative works of, display, perform, and distribute the Software and make,
 * use, sell, offer for sale, import, export, have made, and have sold the
 * Software and the Larger Work(s), and to sublicense the foregoing rights on
 * either these or other terms.
 *
 * This license is subject to the following condition:
 *
 * The above copyright notice and either this complete permission notice or at a
 * minimum a reference to the UPL must be included in all copies or substantial
 * portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package com.oracle.truffle.nfi.impl;

import com.oracle.truffle.api.dsl.Specialization;
import com.oracle.truffle.api.nodes.Node;
import java.nio.ByteBuffer;

abstract class ClosureArgumentNode extends Node {

    public abstract Object execute(Object arg);

    abstract static class BufferClosureArgumentNode extends ClosureArgumentNode {

        private final LibFFIType type;

        BufferClosureArgumentNode(LibFFIType type) {
            this.type = type;
        }

        @Specialization
        public Object deserialize(ByteBuffer arg) {
            NativeArgumentBuffer buffer = new NativeArgumentBuffer.Direct(arg, 0);
            return type.deserialize(buffer);
        }
    }

    static class ObjectClosureArgumentNode extends ClosureArgumentNode {

        @Override
        public Object execute(Object arg) {
            if (arg == null) {
                return new NativePointer(0);
            } else {
                return arg;
            }
        }
    }

    static class StringClosureArgumentNode extends ClosureArgumentNode {

        @Override
        public Object execute(Object arg) {
            if (arg == null) {
                return new NativeString(0);
            } else {
                return arg;
            }
        }
    }
}

==> truffle-nfi-19.2.0.1-sources.jar: com/oracle/truffle/nfi/impl/ClosureArgumentNode.java <==
/*
 * Copyright (c) 2017, 2019, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * The Universal Permissive License (UPL), Version 1.0
 *
 * Subject to the condition set forth below, permission is hereby granted to any
 * person obtaining a copy of this software, associated documentation and/or
 * data (collectively the "Software"), free of charge and under any and all
 * copyright rights in the Software, and any and all patent rights owned or
 * freely licensable by each licensor hereunder covering either (i) the
 * unmodified Software as contributed to or provided by such licensor, or (ii)
 * the Larger Works (as defined below), to deal in both
 *
 * (a) the Software, and
 *
 * (b) any piece of software and/or hardware listed in the lrgrwrks.txt file if
 * one is included with the Software each a "Larger Work" to which the Software
 * is contributed by such licensors),
 *
 * without restriction, including without limitation the rights to copy, create
 * derivative works of, display, perform, and distribute the Software and make,
 * use, sell, offer for sale, import, export, have made, and have sold the
 * Software and the Larger Work(s), and to sublicense the foregoing rights on
 * either these or other terms.
 *
 * This license is subject to the following condition:
 *
 * The above copyright notice and either this complete permission notice or at a
 * minimum a reference to the UPL must be included in all copies or substantial
 * portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package com.oracle.truffle.nfi.impl;

import com.oracle.truffle.api.CompilerDirectives;
import com.oracle.truffle.api.CompilerDirectives.CompilationFinal;
import com.oracle.truffle.api.TruffleLanguage.LanguageReference;
import com.oracle.truffle.api.nodes.Node;
import java.nio.ByteBuffer;

abstract class ClosureArgumentNode extends Node {

    public abstract Object execute(Object arg);

    static class BufferClosureArgumentNode extends ClosureArgumentNode {

        private final LibFFIType type;
        @Child NativeArgumentLibrary nativeArguments;

        BufferClosureArgumentNode(LibFFIType type) {
            this.type = type;
            this.nativeArguments = NativeArgumentLibrary.getFactory().create(type);
        }

        @Override
        public Object execute(Object arg) {
            NativeArgumentBuffer buffer = new NativeArgumentBuffer.Direct((ByteBuffer) arg, 0);
            return nativeArguments.deserialize(type, buffer);
        }
    }

    static class ObjectClosureArgumentNode extends ClosureArgumentNode {

        @CompilationFinal LanguageReference<NFILanguageImpl> langRef;

        @Override
        public Object execute(Object arg) {
            if (arg == null) {
                if (langRef == null) {
                    CompilerDirectives.transferToInterpreterAndInvalidate();
                    langRef = lookupLanguageReference(NFILanguageImpl.class);
                }
                return NativePointer.create(langRef.get(), 0);
            } else {
                return arg;
            }
        }
    }

    static class StringClosureArgumentNode extends ClosureArgumentNode {

        @Override
        public Object execute(Object arg) {
            if (arg == null) {
                return new NativeString(0);
            } else {
                return arg;
            }
        }
    }
}
